/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
package org.jboss.soa.esb.samples.quickstart.async;

import java.net.URISyntaxException;

import org.jboss.internal.soa.esb.addressing.helpers.EPRHelper;
import org.jboss.soa.esb.UnmarshalException;
import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.addressing.Call;
import org.jboss.soa.esb.addressing.EPR;
import org.jboss.soa.esb.addressing.MalformedEPRException;
import org.jboss.soa.esb.couriers.CourierException;
import org.jboss.soa.esb.couriers.CourierUtil;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;

public class ContinuationResponseAction extends AbstractActionPipelineProcessor
{
	public ContinuationResponseAction(final ConfigTree configTree)
	{
	}
	
	public Message process(final Message message)
		throws ActionProcessingException
	{
		final Call call = message.getHeader().getCall() ;
		final EPR toEPR = call.getTo() ;
		final String continuationReplyToValue = toEPR.getAddr().getExtensionValue("replyTo") ;
		final EPR continuationReplyTo ;
		try
		{
			continuationReplyTo = EPRHelper.fromXMLString(continuationReplyToValue) ;
		}
		catch (final UnmarshalException ue)
		{
			throw new ActionProcessingException("Count not deserialise continuation reply to EPR", ue) ;
		}
		
		call.setTo(continuationReplyTo) ;
		call.setFrom(null) ;
		
		try
		{
			CourierUtil.deliverMessage(message);
		}
		catch (final URISyntaxException urise)
		{
			throw new ActionProcessingException("Failed to deliver continuation response", urise) ;
		}
		catch (final MalformedEPRException mepre)
		{
			throw new ActionProcessingException("Failed to deliver continuation response", mepre) ;
		}
		catch (final CourierException ce)
		{
			throw new ActionProcessingException("Failed to deliver continuation response", ce) ;
		}

		return null;
	}
}