/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
package org.jboss.soa.esb.samples.quickstart.async;

import org.jboss.internal.soa.esb.addressing.helpers.EPRHelper;
import org.jboss.soa.esb.ConfigurationException;
import org.jboss.soa.esb.MarshalException;
import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.addressing.Call;
import org.jboss.soa.esb.addressing.EPR;
import org.jboss.soa.esb.addressing.eprs.LogicalEPR;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;

public class InitContinuationAction extends AbstractActionPipelineProcessor
{
	/**
	 * Continuation category.
	 */
	private final String category ;
	/**
	 * Continuation name.
	 */
	private final String name ;
	
	public InitContinuationAction(final ConfigTree configTree)
		throws ConfigurationException
	{
		category = configTree.getAttribute("continuation-category") ;
		name = configTree.getAttribute("continuation-name") ;
		
		if ((category == null) || (name == null))
		{
			throw new ConfigurationException("Missing category/name property for continuation") ;
		}
	}
	
	public Message process(final Message message)
		throws ActionProcessingException
	{
		final Call call = message.getHeader().getCall() ;
		final EPR replyTo = call.getReplyTo() ;
		final LogicalEPR continuationReplyTo = new LogicalEPR(category, name) ;
		final String replyToEPRValue ;
		try
		{
			replyToEPRValue = EPRHelper.toXMLString(replyTo) ;
		}
		catch (final MarshalException me)
		{
			throw new ActionProcessingException("Could not process the replyTo EPR", me) ;
		}
		continuationReplyTo.getAddr().addExtension("replyTo", replyToEPRValue) ;
		call.setReplyTo(continuationReplyTo) ;
		return message ;
	}
}